package cn.palmte.work.controller.backend;

import cn.palmte.work.bean.ResponseMsg;
import cn.palmte.work.config.Constant;
import cn.palmte.work.model.Admin;
import cn.palmte.work.pojo.ModifyPasswordRequest;
import cn.palmte.work.service.AccountService;
import cn.palmte.work.service.AdminService;
import cn.palmte.work.service.SysRoleService;
import cn.palmte.work.utils.InterfaceUtil;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import top.jfunc.common.crypto.symmetric.DESAS;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Controller
@RequestMapping("/account")
public class AccountController extends BaseController {
    private static final Logger logger = LoggerFactory.getLogger(AccountController.class);

    @Autowired
    private AccountService accountService;

    @Autowired
    private AdminService adminService;

    @Autowired
    private SysRoleService sysRoleService;

    @RequestMapping(value = "/updatePassword")
    @ResponseBody
    public ResponseMsg updatePassword(HttpServletRequest request, ModifyPasswordRequest modifyPasswordRequest){
        ResponseMsg responseMsg = new ResponseMsg();
        try{
            String privateKey = (String) request.getSession().getAttribute(Constant.PRIVATEKEY);
            adminService.updatePassword(modifyPasswordRequest.getId(), modifyPasswordRequest.getPassword(),
                modifyPasswordRequest.getNewPassword(), privateKey);
            responseMsg.setStatus(0);
            responseMsg.setMsg("密码修改成功");
        }
        catch(Exception e){
            responseMsg.setStatus(1);
            responseMsg.setMsg("密码修改失败："+e.getMessage());
        }
        return responseMsg;
    }

    @RequestMapping("/list")
    public String list(@RequestParam(value = KEYWORDS, required = false) String keywords,
                       @RequestParam(value = PAGE_NUMBER, defaultValue = DEFAULT_PAGE_NUMBER) int pageNumber,
                       @RequestParam(value = PAGE_SIZE, defaultValue = DEFAULT_PAGE_SIZE) int pageSize,
                       Map<String, Object> model) {
        // 初始化参数
        setModel(keywords, model);
        ConcurrentHashMap<String, String> searchInfo = getSearchInfo(keywords, model);
        model.put("roleList", sysRoleService.getAllEnableSysRole());
        model.put("pager", accountService.getAdminList(searchInfo, pageSize, pageNumber));
        return "/admin/account_list";
    }

    /**
     * 保存或更新用户
     */
    @RequestMapping("/save")
    public String save(@RequestParam("userId") int userId,Admin admin, HttpServletRequest request, Map<String, Object> model) throws IOException {
        if(admin.getRoleId()<=0){
            model.put("errorMessage","角色不能为空！");
            return "/common/error";
        }
        //获取验证用户信息
        String message = accountService.validateUserExistInfo(userId,admin);
        if(StringUtils.isNotEmpty(message)){
            model.put("errorMessage",message);
            return "/common/error";
        }

     try {
            HttpSession session = request.getSession();
            String privateKey = (String) session.getAttribute(Constant.PRIVATEKEY);

            accountService.saveOrUpdateAccount(userId,admin.getRoleId(),admin, privateKey);
        } catch (Exception e) {
            logger.error("保存账号出错！" + e.toString());
            return "新增账号出错!请联系管理员处理！";
        }
        return "redirect:/account/list";
    }

    /**
     * 跳转到新增页面
     */
    @RequestMapping("/add")
    public String add(HttpServletRequest request, Map<String, Object> model) {
        genRSAKeyPair(request, model);
        model.put("userId", -1);
        model.put("account", new Admin());
        //获取所有角色
        model.put("roleList", sysRoleService.getAllEnableSysRole());
        return "/admin/account_input";
    }

    /**
     * 启用、禁用用户
     */
    @RequestMapping("/changeStatus")
    public ResponseMsg changeStatus(Admin admin) {
        boolean locked = accountService.changeStatus(admin.getId(),admin.getEnabled());
        if (locked) {
            return ResponseMsg.buildSuccessMsg("操作成功");
        } else {
            return ResponseMsg.buildFailedMsg("操作失败");
        }
    }

    /**
     * 根据ID查询用户信息编辑
     */
    @RequestMapping("/edit")
    public String edit(@RequestParam("userId") int userId, Map<String, Object> model) {
        model.put("userId", userId);
        Admin admin = accountService.findUserById(userId);
        model.put("roleId", String.valueOf(admin.getRoleId()));
        model.put("roleType", admin.getRoleType());
        model.put("roleLevel", admin.getRoleLevel());
        model.put("realName", admin.getRealName());
        model.put("userName", admin.getUserName());
        model.put("telephone", admin.getTelephone());
        model.put("regionId", admin.getRegionId());
        model.put("account", admin);
        //获取所有角色
        model.put("roleList", sysRoleService.getAllEnableSysRole());
        return "/admin/account_input";
    }

    /**
     * 根据删除用户
     */
    @RequestMapping("/delete")
    @ResponseBody
    public ResponseMsg delete(@RequestParam("userId") int userId) {
        ResponseMsg responseMsg = new ResponseMsg();
        boolean deleted = accountService.deleteAccount(userId);

        if (deleted) {
            responseMsg.setStatus(0);
            responseMsg.setMsg("删除成功");
        } else {
            responseMsg.setStatus(1);
            responseMsg.setMsg("删除失败");
        }
        return responseMsg;
    }

    /**
     * 根据ID重置用户密码
     */
    @RequestMapping("/resetPassword")
    @ResponseBody
    public ResponseMsg resetPassword(@RequestParam("userId") int userId,HttpServletRequest request) {
        HttpSession session = request.getSession();
        String privateKey = (String) session.getAttribute(Constant.PRIVATEKEY);
        boolean flag = accountService.resetPassword(userId,privateKey);
        ResponseMsg responseMsg = new ResponseMsg();
        if (flag) {
            responseMsg.setStatus(0);
            responseMsg.setMsg("重置密码成功");
        } else {
            responseMsg.setStatus(1);
            responseMsg.setMsg("重置密码失败");
        }
        return responseMsg;
    }

    /**
     * 修改密码请求
     */
    @RequestMapping("/password")
    public String password(HttpServletRequest request,Map<String, Object> model){
        Admin admin = adminService.getAdminById(InterfaceUtil.getAdminId());
        try {
            String id = new DESAS().encrypt(admin.getId().toString());
            model.put("userId", id);
        } catch (Exception e) {
            e.printStackTrace();
        }
        model.put("SysUser", admin);
        genRSAKeyPair(request,model);
        return "/admin/account_password";

    }

    private void setModel(@RequestParam(value = "keywords", required = false) String keywords, Map<String, Object> model) {
        model.put("roleType", "-1");
        model.put("keywords", keywords);
    }



}
